<?php
namespace Magenest\Salesforce\Controller\Adminhtml\Sync;

use Magenest\Salesforce\Model\Sync;
use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;

/**
 * Class Contact
 * @package Magenest\Salesforce\Controller\Adminhtml\Sync
 */
class Contact extends Action
{
    /**
     * @var Sync\Contact
     */
    protected $_contact;

    /**
     * Customer constructor.
     * @param Context $context
     * @param Sync\Contact $contact
     */
    public function __construct(
        Context $context,
        Sync\Contact $contact
    ) {
        $this->_contact = $contact;
        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $customerId = $this->getRequest()->getParam('id');
        try {
            if ($customerId) {
                $this->_contact->sync($customerId, true);
                $this->messageManager->addSuccessMessage(
                    __('Contact sync process complete, check out Reports for result.')
                );
            } else {
                $this->messageManager->addNoticeMessage(
                    __('No contact has been selected')
                );
            }
        } catch (\Throwable $e) {
            $this->messageManager->addErrorMessage(
                __('Something happen during syncing process. Detail: ' . $e->getMessage() . ". Trace: " . $e->getTraceAsString())
            );
        }
        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        $resultRedirect->setPath('customer/index/edit', ['id' => $customerId]);
        return $resultRedirect;
    }

    /**
     * @return bool
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Magenest_Salesforce::config_salesforce');
    }
}
