<?php
namespace Magenest\Salesforce\Controller\Oauth;

use Magenest\Salesforce\Helper\Constant;
use Magenest\Salesforce\Helper\Data as HelperData;
use Magenest\Salesforce\Logger\Logger;
use Magenest\Salesforce\Model\Connector;
use Magenest\Salesforce\Model\Sync\Product;
use Magento\Config\Model\ResourceModel\Config as ConfigResourceModel;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\App\Cache\Type\Config;
use Magento\Framework\App\Cache\Manager;
use Magento\Backend\Model\Url as AdminUrl;
use Magento\Framework\Controller\ResultFactory;

/**
 * Class Callback
 * @package Magenest\Salesforce\Controller\Oauth
 */
class Callback extends \Magento\Framework\App\Action\Action
{
    /** @var HelperData  */
    protected $_helperData;

    /** @var Connector  */
    protected $_connector;

    /** @var Logger  */
    protected $_logger;

    /** @var Json  */
    protected $_jsonFramework;

    /** @var ConfigResourceModel  */
    protected $_resourceConfig;

    /** @var Manager  */
    protected $_cacheManager;

    /** @var AdminUrl  */
    protected $_adminUrl;

    /** @var ResultFactory  */
    protected $_resultFactory;

    /** @var Product  */
    protected $_syncProduct;

    /**
     * Callback constructor.
     * @param HelperData $helperData
     * @param Connector $connector
     * @param Logger $logger
     * @param Product $syncProduct
     * @param Json $jsonFramework
     * @param ConfigResourceModel $configResourceModel
     * @param Manager $cacheManager
     * @param AdminUrl $adminUrl
     * @param ResultFactory $resultFactory
     * @param Context $context
     */
    public function __construct(
        HelperData $helperData,
        Connector $connector,
        Logger $logger,
        Product $syncProduct,
        Json $jsonFramework,
        ConfigResourceModel $configResourceModel,
        Manager $cacheManager,
        AdminUrl $adminUrl,
        ResultFactory $resultFactory,
        Context $context
    ) {
        $this->_helperData = $helperData;
        $this->_connector = $connector;
        $this->_logger = $logger;
        $this->_syncProduct = $syncProduct;
        $this->_jsonFramework = $jsonFramework;
        $this->_resourceConfig = $configResourceModel;
        $this->_cacheManager = $cacheManager;
        $this->_adminUrl = $adminUrl;
        $this->_resultFactory = $resultFactory;
        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        try {
            $params =  $this->getRequest()->getParams();
            if (isset($params['code']) && $code = $params['code']) {
                $connectEnvironment = $this->_helperData->getConnectEnvironment();
                $instantUrl = Constant::INSTANCE_SF_SANDBOX_MODE;
                if ($connectEnvironment === 1 || $connectEnvironment === "1") {
                    $instantUrl = Constant::INSTANCE_SF_PRODUCTION_MODE;
                }
                $clientId = $this->_helperData->getConfigurationByPath(Connector::XML_PATH_SALESFORCE_CLIENT_ID);
                $clientSecret = $this->_helperData->getConfigurationByPath(Connector::XML_PATH_SALESFORCE_CLIENT_SECRET);
                $redirectUri = rtrim($this->_helperData->getFrontendUrlByPath("salesforce/oauth/callback"), "/");
                $url = $instantUrl . 'services/oauth2/token';
                $params   = [
                    'grant_type'    => 'authorization_code',
                    'code' => $code,
                    'client_id'     => $clientId,
                    'client_secret' => $clientSecret,
                    'redirect_uri'      => $redirectUri
                ];
                $response = $this->_connector->makeRequest(\Magenest\Salesforce\Model\Connector::POST, $url, [], $params);
                $response = $this->_jsonFramework->unserialize($response);
                if (is_array($response) && !empty($response) && isset($response['access_token']) && isset($response['instance_url']) && isset($response['refresh_token'])) {
                    $this->_helperData->saveConfig(Connector::XML_PATH_SALESFORCE_INSTANCE_URL, $response['instance_url']);
                    $this->_helperData->saveConfig(Connector::XML_PATH_SALESFORCE_ACCESS_TOKEN, $response['access_token']);
                    $this->_helperData->saveConfig(Connector::XML_PATH_SALESFORCE_REFRESH_TOKEN, $response['refresh_token']);
                    $this->_helperData->saveConfig(Connector::XML_PATH_SALESFORCE_IS_CONNECTED, 1);
                    $this->cleanConfig();
                    $this->syncDefaultData();
                    $this->messageManager->addSuccessMessage(__("Success"));
                } else {
                    throw new \Exception($response['error_description']);
                }
            }
        } catch (\Throwable $exception) {
            $this->_logger->critical($exception->getMessage());
            $this->messageManager->addErrorMessage($exception->getMessage());
        }
        //$configurationUrl = $this->_adminUrl->getUrl(Connector::SALESFORCE_CONFIGURATION_SECTION);
        return $this->_resultFactory->create(ResultFactory::TYPE_PAGE);
    }

    /**
     * @return void
     */
    private function cleanConfig()
    {
        $this->_cacheManager->clean([Config::TYPE_IDENTIFIER]);
    }

    /**
     * @throws \Exception
     */
    private function syncDefaultData()
    {
        try {
            $this->_syncProduct->syncShippingProduct();
            $this->_syncProduct->syncTaxProduct();
            $this->_syncProduct->syncDiscount();
        } catch (\Throwable $e) {
            throw new \Exception($e->getMessage());
        }
    }
}
