<?php
namespace Magenest\Salesforce\Setup\Patch\Data;

use Magenest\Salesforce\Model\Connector;
use Magento\Config\Model\ResourceModel\Config as ResourceConfig;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;
use Magento\Framework\Setup\Patch\PatchVersionInterface;

class InstallDefaultValue implements DataPatchInterface, PatchVersionInterface
{
    /**
     * @var ModuleDataSetupInterface $moduleDataSetup
     */
    private $moduleDataSetup;

    /**
     * @var ResourceConfig
     */
    private $resourceConfig;

    /**
     * @param ModuleDataSetupInterface $moduleDataSetup
     * @param ResourceConfig $resourceConfig
     */
    public function __construct(ModuleDataSetupInterface $moduleDataSetup, ResourceConfig $resourceConfig)
    {
        $this->moduleDataSetup = $moduleDataSetup;
        $this->resourceConfig       = $resourceConfig;
    }

    /**
     * Do Upgrade
     *
     * @return void
     */
    public function apply()
    {
        $setup = $this->moduleDataSetup;
        $data = [
            'scope' => 'default',
            'scope_id' => 0,
            'path' => Connector::XML_PATH_SALESFORCE_IS_CONNECTED,
            'value' => 0,
        ];
        $setup->getConnection()
            ->insertOnDuplicate($setup->getTable('core_config_data'), $data, ['value']);
        $this->resourceConfig->saveConfig(Connector::XML_STANDARD_PRICE_BOOK_ID_PATH, null, 'default', 0);
    }

    /**
     * @inheritdoc
     */
    public function getAliases()
    {
        return [];
    }

    /**
     * @inheritdoc
     */
    public static function getDependencies()
    {
        return [];
    }

    /**
     * @inheritdoc
     */
    public static function getVersion()
    {
        return '3.3.0';
    }
}
